"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.makeAvailable = exports.removePrefix = exports.removeObject = exports.storeObject = exports.buildKey = void 0;
const tslib_1 = require("tslib");
const fs_extra_1 = require("fs-extra");
const path_1 = require("path");
const client_s3_1 = require("@aws-sdk/client-s3");
const lib_storage_1 = require("@aws-sdk/lib-storage");
const core_utils_1 = require("../../../helpers/core-utils");
const misc_1 = require("../../../helpers/custom-validators/misc");
const logger_1 = require("../../../helpers/logger");
const config_1 = require("../../../initializers/config");
const urls_1 = require("../urls");
const client_1 = require("./client");
const logger_2 = require("./logger");
function storeObject(options) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const { inputPath, objectStorageKey, bucketInfo } = options;
        logger_1.logger.debug('Uploading file %s to %s%s in bucket %s', inputPath, bucketInfo.PREFIX, objectStorageKey, bucketInfo.BUCKET_NAME, (0, logger_2.lTags)());
        const fileStream = (0, fs_extra_1.createReadStream)(inputPath);
        return uploadToStorage({ objectStorageKey, content: fileStream, bucketInfo });
    });
}
exports.storeObject = storeObject;
function removeObject(filename, bucketInfo) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const command = new client_s3_1.DeleteObjectCommand({
            Bucket: bucketInfo.BUCKET_NAME,
            Key: buildKey(filename, bucketInfo)
        });
        return (0, client_1.getClient)().send(command);
    });
}
exports.removeObject = removeObject;
function removePrefix(prefix, bucketInfo) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const s3Client = (0, client_1.getClient)();
        const commandPrefix = bucketInfo.PREFIX + prefix;
        const listCommand = new client_s3_1.ListObjectsV2Command({
            Bucket: bucketInfo.BUCKET_NAME,
            Prefix: commandPrefix
        });
        const listedObjects = yield s3Client.send(listCommand);
        if ((0, misc_1.isArray)(listedObjects.Contents) !== true) {
            const message = `Cannot remove ${commandPrefix} prefix in bucket ${bucketInfo.BUCKET_NAME}: no files listed.`;
            logger_1.logger.error(message, Object.assign({ response: listedObjects }, (0, logger_2.lTags)()));
            throw new Error(message);
        }
        for (const object of listedObjects.Contents) {
            const command = new client_s3_1.DeleteObjectCommand({
                Bucket: bucketInfo.BUCKET_NAME,
                Key: object.Key
            });
            yield s3Client.send(command);
        }
        if (listedObjects.IsTruncated)
            yield removePrefix(prefix, bucketInfo);
    });
}
exports.removePrefix = removePrefix;
function makeAvailable(options) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const { key, destination, bucketInfo } = options;
        yield (0, fs_extra_1.ensureDir)((0, path_1.dirname)(options.destination));
        const command = new client_s3_1.GetObjectCommand({
            Bucket: bucketInfo.BUCKET_NAME,
            Key: buildKey(key, bucketInfo)
        });
        const response = yield (0, client_1.getClient)().send(command);
        const file = (0, fs_extra_1.createWriteStream)(destination);
        yield (0, core_utils_1.pipelinePromise)(response.Body, file);
        file.close();
    });
}
exports.makeAvailable = makeAvailable;
function buildKey(key, bucketInfo) {
    return bucketInfo.PREFIX + key;
}
exports.buildKey = buildKey;
function uploadToStorage(options) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const { content, objectStorageKey, bucketInfo } = options;
        const input = {
            Body: content,
            Bucket: bucketInfo.BUCKET_NAME,
            Key: buildKey(objectStorageKey, bucketInfo)
        };
        if (config_1.CONFIG.OBJECT_STORAGE.UPLOAD_ACL) {
            input.ACL = config_1.CONFIG.OBJECT_STORAGE.UPLOAD_ACL;
        }
        const parallelUploads3 = new lib_storage_1.Upload({
            client: (0, client_1.getClient)(),
            queueSize: 4,
            partSize: config_1.CONFIG.OBJECT_STORAGE.MAX_UPLOAD_PART,
            leavePartsOnError: true,
            params: input
        });
        const response = (yield parallelUploads3.done());
        if (!response.Bucket) {
            const message = `Error uploading ${objectStorageKey} to bucket ${bucketInfo.BUCKET_NAME}`;
            logger_1.logger.error(message, Object.assign({ response }, (0, logger_2.lTags)()));
            throw new Error(message);
        }
        logger_1.logger.debug('Completed %s%s in bucket %s', bucketInfo.PREFIX, objectStorageKey, bucketInfo.BUCKET_NAME, (0, logger_2.lTags)());
        return (0, urls_1.getPrivateUrl)(bucketInfo, objectStorageKey);
    });
}
//# sourceMappingURL=object-storage-helpers.js.map