"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const chai_1 = require("chai");
const server_commands_1 = require("../../../../shared/server-commands");
describe('Test services', function () {
    let server = null;
    let playlistUUID;
    let playlistDisplayName;
    let video;
    const urlSuffixes = [
        {
            input: '',
            output: ''
        },
        {
            input: '?param=1',
            output: ''
        },
        {
            input: '?muted=1&warningTitle=0&toto=1',
            output: '?muted=1&warningTitle=0'
        }
    ];
    before(function () {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            this.timeout(30000);
            server = yield (0, server_commands_1.createSingleServer)(1);
            yield (0, server_commands_1.setAccessTokensToServers)([server]);
            yield (0, server_commands_1.setDefaultVideoChannel)([server]);
            {
                const attributes = { name: 'my super name' };
                yield server.videos.upload({ attributes });
                const { data } = yield server.videos.list();
                video = data[0];
            }
            {
                const created = yield server.playlists.create({
                    attributes: {
                        displayName: 'The Life and Times of Scrooge McDuck',
                        privacy: 1,
                        videoChannelId: server.store.channel.id
                    }
                });
                playlistUUID = created.uuid;
                playlistDisplayName = 'The Life and Times of Scrooge McDuck';
                yield server.playlists.addElement({
                    playlistId: created.id,
                    attributes: {
                        videoId: video.id
                    }
                });
            }
        });
    });
    it('Should have a valid oEmbed video response', function () {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            for (const basePath of ['/videos/watch/', '/w/']) {
                for (const suffix of urlSuffixes) {
                    const oembedUrl = server.url + basePath + video.uuid + suffix.input;
                    const res = yield server.services.getOEmbed({ oembedUrl });
                    const expectedHtml = '<iframe width="560" height="315" sandbox="allow-same-origin allow-scripts allow-popups" ' +
                        `title="${video.name}" src="http://${server.host}/videos/embed/${video.uuid}${suffix.output}" ` +
                        'frameborder="0" allowfullscreen></iframe>';
                    const expectedThumbnailUrl = 'http://' + server.host + video.previewPath;
                    (0, chai_1.expect)(res.body.html).to.equal(expectedHtml);
                    (0, chai_1.expect)(res.body.title).to.equal(video.name);
                    (0, chai_1.expect)(res.body.author_name).to.equal(server.store.channel.displayName);
                    (0, chai_1.expect)(res.body.width).to.equal(560);
                    (0, chai_1.expect)(res.body.height).to.equal(315);
                    (0, chai_1.expect)(res.body.thumbnail_url).to.equal(expectedThumbnailUrl);
                    (0, chai_1.expect)(res.body.thumbnail_width).to.equal(850);
                    (0, chai_1.expect)(res.body.thumbnail_height).to.equal(480);
                }
            }
        });
    });
    it('Should have a valid playlist oEmbed response', function () {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            for (const basePath of ['/videos/watch/playlist/', '/w/p/']) {
                for (const suffix of urlSuffixes) {
                    const oembedUrl = server.url + basePath + playlistUUID + suffix.input;
                    const res = yield server.services.getOEmbed({ oembedUrl });
                    const expectedHtml = '<iframe width="560" height="315" sandbox="allow-same-origin allow-scripts allow-popups" ' +
                        `title="${playlistDisplayName}" src="http://${server.host}/video-playlists/embed/${playlistUUID}${suffix.output}" ` +
                        'frameborder="0" allowfullscreen></iframe>';
                    (0, chai_1.expect)(res.body.html).to.equal(expectedHtml);
                    (0, chai_1.expect)(res.body.title).to.equal('The Life and Times of Scrooge McDuck');
                    (0, chai_1.expect)(res.body.author_name).to.equal(server.store.channel.displayName);
                    (0, chai_1.expect)(res.body.width).to.equal(560);
                    (0, chai_1.expect)(res.body.height).to.equal(315);
                    (0, chai_1.expect)(res.body.thumbnail_url).exist;
                    (0, chai_1.expect)(res.body.thumbnail_width).to.equal(280);
                    (0, chai_1.expect)(res.body.thumbnail_height).to.equal(157);
                }
            }
        });
    });
    it('Should have a valid oEmbed response with small max height query', function () {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            for (const basePath of ['/videos/watch/', '/w/']) {
                const oembedUrl = 'http://' + server.host + basePath + video.uuid;
                const format = 'json';
                const maxHeight = 50;
                const maxWidth = 50;
                const res = yield server.services.getOEmbed({ oembedUrl, format, maxHeight, maxWidth });
                const expectedHtml = '<iframe width="50" height="50" sandbox="allow-same-origin allow-scripts allow-popups" ' +
                    `title="${video.name}" src="http://${server.host}/videos/embed/${video.uuid}" ` +
                    'frameborder="0" allowfullscreen></iframe>';
                (0, chai_1.expect)(res.body.html).to.equal(expectedHtml);
                (0, chai_1.expect)(res.body.title).to.equal(video.name);
                (0, chai_1.expect)(res.body.author_name).to.equal(server.store.channel.displayName);
                (0, chai_1.expect)(res.body.height).to.equal(50);
                (0, chai_1.expect)(res.body.width).to.equal(50);
                (0, chai_1.expect)(res.body).to.not.have.property('thumbnail_url');
                (0, chai_1.expect)(res.body).to.not.have.property('thumbnail_width');
                (0, chai_1.expect)(res.body).to.not.have.property('thumbnail_height');
            }
        });
    });
    after(function () {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield (0, server_commands_1.cleanupTests)([server]);
        });
    });
});
//# sourceMappingURL=services.js.map